
/** Represents a Visitor (external function) for a binary search tree.
 *   It is parametrized by the type of elements in the tree, and the
 *   type that will be returned. */
public interface ABSTVisitor<T, R>{
    
    /** Compute a result for a Leaf of a BST */
    public R visitLeaf();
    
    /** Compute a result for a Node of a BST */
    public R visitNode(T data, ABST<T> left, ABST<T> right);
}


/** Compute the number of Nodes in a BST */
class CountNodes<T> implements ABSTVisitor<T, Integer>{
    
    /** Count the number of Nodes in a Leaf */
    public Integer visitLeaf(){
        return 0;
    }
    
    /** Count the number of Nodes in a Node */
    public Integer visitNode(T data, ABST<T> left, ABST<T> right){
        return 1 + left.accept(this) + right.accept(this);
    }
}
